package dev.adriankuta.flights.ui.home

import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.Spacer
import androidx.compose.foundation.layout.fillMaxWidth
import androidx.compose.foundation.layout.height
import androidx.compose.foundation.layout.padding
import androidx.compose.material3.Text
import androidx.compose.runtime.Composable
import androidx.compose.runtime.getValue
import androidx.compose.ui.Modifier
import androidx.compose.ui.unit.dp
import androidx.hilt.navigation.compose.hiltViewModel
import androidx.lifecycle.compose.collectAsStateWithLifecycle
import dev.adriankuta.flights.domain.types.AirportInfo
import dev.adriankuta.flights.domain.types.City
import dev.adriankuta.flights.domain.types.Coordinates
import dev.adriankuta.flights.domain.types.Country
import dev.adriankuta.flights.domain.types.MacCity
import dev.adriankuta.flights.domain.types.Region
import dev.adriankuta.flights.ui.designsystem.theme.FlightsTheme
import dev.adriankuta.flights.ui.designsystem.theme.PreviewDevices
import dev.adriankuta.flights.ui.home.components.AirportDropdown
import dev.adriankuta.flights.ui.home.components.DatePicker
import java.time.LocalDate

@Composable
internal fun HomeScreen(
    viewModel: HomeScreenViewModel = hiltViewModel(),
) {
    val homeUiState by viewModel.uiState.collectAsStateWithLifecycle()

    HomeScreen(
        uiState = homeUiState,
        onOriginAirportSelect = viewModel::selectOriginAirport,
        onDestinationAirportSelect = viewModel::selectDestinationAirport,
        onDateSelect = viewModel::selectDate,
    )
}

@Composable
private fun HomeScreen(
    uiState: HomeUiState,
    onOriginAirportSelect: (AirportInfo) -> Unit,
    onDestinationAirportSelect: (AirportInfo) -> Unit,
    onDateSelect: (LocalDate) -> Unit,
    modifier: Modifier = Modifier,
) {
    Column(
        modifier = modifier.padding(16.dp),
    ) {
        when (uiState) {
            is HomeUiState.Error -> Text("Error")
            HomeUiState.Loading -> Text("Loading")
            is HomeUiState.Success -> {
                AirportDropdown(
                    label = "Origin Airport",
                    airports = uiState.originAirports,
                    selectedAirport = uiState.selectedOriginAirport,
                    onAirportSelect = onOriginAirportSelect,
                    modifier = Modifier.fillMaxWidth(),
                )

                Spacer(modifier = Modifier.height(16.dp))

                AirportDropdown(
                    label = "Destination Airport",
                    airports = uiState.destinationAirports,
                    selectedAirport = uiState.selectedDestinationAirport,
                    onAirportSelect = onDestinationAirportSelect,
                    modifier = Modifier.fillMaxWidth(),
                    enabled = uiState.selectedOriginAirport != null,
                )

                Spacer(modifier = Modifier.height(16.dp))

                DatePicker(
                    label = "Departure Date",
                    selectedDate = uiState.selectedDate,
                    onDateSelect = onDateSelect,
                    modifier = Modifier.fillMaxWidth(),
                    enabled = uiState.selectedDestinationAirport != null,
                )
            }
        }
    }
}

@PreviewDevices
@Composable
private fun HomeScreenLoadingPreview() {
    FlightsTheme {
        HomeScreen(
            uiState = HomeUiState.Loading,
            onOriginAirportSelect = {},
            onDestinationAirportSelect = {},
            onDateSelect = {},
        )
    }
}

@PreviewDevices
@Composable
private fun HomeScreenSuccessPreview() {
    FlightsTheme {
        val mockAirports = listOf(
            AirportInfo(
                code = "WAW",
                name = "Warsaw Chopin Airport",
                seoName = "warsaw",
                isBase = true,
                timeZone = "Europe/Warsaw",
                city = City("WAW", "Warsaw"),
                macCity = MacCity("WARSAW"),
                region = Region("WARSAW_PL", "Warsaw"),
                country = Country("PL", "Poland", "PLN"),
                coordinates = Coordinates(52.1657, 20.9671),
            ),
            AirportInfo(
                code = "KRK",
                name = "Krakow Airport",
                seoName = "krakow",
                isBase = true,
                timeZone = "Europe/Warsaw",
                city = City("KRK", "Krakow"),
                macCity = MacCity("KRAKOW"),
                region = Region("KRAKOW_PL", "Krakow"),
                country = Country("PL", "Poland", "PLN"),
                coordinates = Coordinates(50.0777, 19.7848),
            ),
        )

        HomeScreen(
            uiState = HomeUiState.Success(
                originAirports = mockAirports,
                destinationAirports = mockAirports,
                selectedOriginAirport = mockAirports.first(),
                selectedDestinationAirport = mockAirports.last(),
                selectedDate = LocalDate.now(),
            ),
            onOriginAirportSelect = {},
            onDestinationAirportSelect = {},
            onDateSelect = {},
        )
    }
}
